;------------------------------------------------------------------------------
;
; NAME:
;
;   make-cm          -- interactively load and optionally save Common Music
;
; SYNTAX:
;
;   make-cm &key <keywords>
;
; DESCRIPTION
;
;   This is the build script for Common Music.  Call it with as many of
;   the keywords below as you like.  Unspecified keywords will be requested at
;   runtime and a default choice presented.  Be sure you have write permission
;   and enough disk space to save the Common Music application. File
;   names and directories should not contain relative paths or links.
;   The keywords are:
;
; KEYWORD           DESCRIPTION                              DEFAULT VALUE
;------------------------------------------------------------------------------
;
; cm-directory      Common Music source directory            NIL
;
; cm-image-name     The application file to build, if any.  <cm-directory>/cm
; 
; site-directory   Directory for site customized files.     <cm-directory>/site
;                  If you specify a directory other than
;                  the default, you must move any scripts
;                  from cm/site that you intend to use to
;                  your personal site directory.
;
; bin-directory     Where to put the binaries if you don't  <cm-directory>
;                   keep them with the sources.
;
; pcl-directory     If you use PCL (public domain CLOS),     NIL      
;                   set this variable to the PCL source 
;                   directory.  You must have compiled PCL 
;                   before building Common Music.
;
; clm-directory     If you use CLM (Common Lisp Music) set   NIL
;                   to the CLM binary directory. You must 
;                   have compiled CLM before building 
;                   Common Music.
;
; cmn-directory     If you use CMN, set to the CMN source 
;                   directory. You must have compiled CMN 
;                   before building Common Music.      
;
; save?             If T, make-cm compiles sources and       :IF-NO-COMPILE
;                   saves the cm application in one pass. 
;                   If :IF-NO-COMPILE, the build script  
;                   either compiles sources or loads the  
;                   binaries and saves the application if 
;                   no sources needed compilation.  If NIL, 
;                   make-cm script compiles/loads sources 
;                   but will not save an application. 
;                   The :IF-NO-COMPILE option results in the 
;                   smallest application size.
;
; syntaxes          The list of output syntaxes to build     '(:MIDI :CSOUND)
;                   into Common Music.  Select any from:
;                    
;                       :MIDI
;                       :CLM 
;                       :CMN 
;                       :CSound 
;                       :MusicKit 
;                       :CMix 
;                       :CMusic
;                       :RT 
;                       :SGIMix
;                       :M4C
;                       
;                   The first syntax in the list becomes  
;                   the default syntax when the system 
;                   boots up.  You must load clm and cmn  
;                   before calling make-cm if you want cm  
;                   to build withto build with either the 
;                   :CLM or :CMN syntax.
;
; capella           Whether to include Capella or not.       T
;                   [MCL only]
;
; extensions        Optional list of additional files to     NIL
;                   load.
;
;------------------------------------------------------------------------------

;[Version Date: 13-Nov-95]

#+cltl2 (in-package :cl-user)
#-cltl2 (in-package :user)
  
(defun make-cm (&key (cm-directory         nil)
		     (cm-image-name        nil)
		     (bin-directory        nil bd?)
		     (pcl-directory        nil pd?)
		     (clm-directory        nil clmd?)
		     (cmn-directory        nil cmnd?) 
		     (site-directory       nil cms?)
		     (save?                 nil si?)
		     (syntaxes            '(:MIDI :CSOUND) sy?)
		     (capella              #+mcl t #-mcl nil cp?)
		     (extensions           nil ex?)
		     &aux 
		     (queried? nil)
		     (compiled? nil)
		     (%syntaxes% '(:MIDI :CLM :CMN :CSound :MusicKit
				   :CMix :CMusic :RT :SGIMix :M4C))
		     (source-type #+(or aclpc dos OS/2 WIN32) "LSP"
				  #-(or aclpc dos OS/2 WIN32) "lisp")
		     (binary-type #+excl "fasl"
                                  #+mcl (pathname-type ccl:*.fasl-pathname*)
				  #+kcl "o"
				  #+aclpc "fsl"
				  #+(and lispworks mips) "mfasl"
				  #+(and clisp (or dos OS/2 WIN32)) "FAS"
				  #+(and clisp (not dos) (not OS/2)
					       (not WIN32)) "fas"
				  #+cmu
				  (c:backend-fasl-file-type c:*backend*))
		     (delimiter
		      (let ((n (namestring #+excl (excl:current-directory)
					   #-excl (truename ""))))
			(subseq n (1- (length n))))))
  ;;; utility functions.
  (labels ((notify (&rest args) (apply #'format t args))
	   (query (question &key default-value test-function 
			    (help-string "No help available.")
			    feedback-string 
			    (reject-string "~S is not valid input.")
			    (fresh-line t)
			    &aux result done)
	     (unless queried?
	       (clear-input #+gcl t)
	       (notify "~2%*** Make CM (Type ? for help, EXIT to exit)~%")
	       (setf queried? t))
	     (block ask
	       (loop
		 (when done (return-from ask)) 
		 #-aclpc
         (progn
           (if fresh-line
		     (notify "~%~A ~@[(<cr>=~A) ~]" question default-value)
		     (notify "~A ~@[(<cr>=~A) ~]" question default-value))
		   (setf result (read-line t nil nil nil)))
         #+aclpc 
         (multiple-value-bind (ans flg button)
           (cg:ask-user-for-string 
             (format nil "~A  ~@[(OK=~A) ~]" question default-value)
             (or default-value "") "EXIT" "OK")
             (declare (ignore flg)) 
           (setf result (if (string-equal button "EXIT") button ans)))
		 (when (string-equal result "")
		   (setf result default-value))
		 (cond ((string-equal result "?")
			(notify help-string))
		       ((string-equal result "exit")
			(return-from make-cm))
		       ((not test-function)
			(setf done t))
		       (t
			(let ((l (multiple-value-list
				  (funcall test-function result))))
			  (if (= (length l) 2)
			    (setf done (first l) result (second l))
			    (setf done (first l) result done))
			  (unless done
			    (notify reject-string result))
			  result)))))
	     (when feedback-string
	       (notify "*** ~A~%" (format nil feedback-string result)))
	     result)
	   (y-or-n-query (prompt &rest args)
	     (apply #'query 
		    (format nil "~A (y or n)" prompt)
		    :default-value "y" :reject-string "~S is not Y or N."
		    :test-function
		    #'(lambda (r &aux (l (length r)))
			(if (string-equal r "YES" :end2 (min l 3))
			  (values t t)
			  (if (string-equal r "NO" :end2 (min l 2))
			    (values t nil)
			    (values nil r))))
		    args))
	   (ensure-directory (dir)
	     #+(or clisp cmucl)
	     ;;  dirs in clisp and cmu must end with delim 
	     (unless (string= (subseq dir (1- (length dir))) delimiter)
	       (setf dir (concatenate 'string dir delimiter)))
	     (let ((n (file-namestring dir)))
	       (if (string= n "")
		 dir
		 (namestring
		  (if (string= n dir)
		    (make-pathname :directory dir)
		    (make-pathname :host (pathname-host dir)
				   :device (pathname-device dir)
				   :directory (append (pathname-directory dir)
						      (list n))))))))
	   (directory-exists? (dir)
	     (setf dir (ensure-directory dir))
	     (if #-(or clisp aclpc) (probe-file dir)
             #+clisp (directory dir)
             #+aclpc (acl:directory-exists-p dir)
		 (values t (namestring (truename dir)))
		 (values nil dir)) )
	   (test-output-file-directory (p)
	     (and (directory-exists?
		   (namestring (make-pathname
				:host (pathname-host p)
				:device (pathname-device p)
				:directory (pathname-directory p))))
		  p))
	   (filename (&rest path)
	     #+(and clisp (or dos win32))
	     (setf path (mapcar #'string-upcase path))
	     (let ((directory (butlast path))
		   (file (car (last path))))
	       #+(or aclpc dos)
	       (when (> (length file) 8)
		 (setf file (subseq file 0 8)))
	       (if directory
		 (let ((m (subseq (first directory) 
				  (1- (length (first directory))))))
		   (apply #'concatenate 'string 
			  (append (cons (car directory) 
					(mapcan #'(lambda (d) (list d m))
						(cdr directory)))
				  (list file))))
		 file))) 
	   (dofile (&rest args)
	     (let* ((sargs (apply #'filename args))
		    (bargs (if bin-directory
			     (apply #'filename (cons bin-directory
						     (last args)))
			     sargs))
		    (source (make-pathname
			     :defaults sargs
			     :type (or (pathname-type sargs) 
				       source-type)))
		    (binary (if bin-directory
			      (make-pathname :defaults bargs :type binary-type)
			      (make-pathname :defaults sargs 
					     :type binary-type)))
		    (source? (probe-file source)) 
		    (binary? (probe-file binary)))
	       (if (and source? (or (not binary?)
				    (< (file-write-date binary)
				       (file-write-date source))))
		 (progn
		   #+mcl (compile-file source :output-file binary 
				       :verbose t)
		   #-mcl (compile-file source :output-file binary)
		   #+clisp ; clean up compiler doo doo file
		   (let ((stupid (make-pathname 
				  :defaults bargs
				  :type #+dos "LIB" #-dos "lib")))
		     (when (probe-file stupid) (delete-file stupid)))
		   (setf compiled? t)
		   (load binary :verbose t))
		 (if binary?
		   (progn
		     (unless source?
		       (warn ";No source file found for \"~S\"" binary))
		     (load binary :verbose t))
		   (cerror "continue building Common Music" 
			   "File not found: ~s" sargs)))))
	   (shell (string) 
	     #+mcl (declare (ignore string))
	     #-unix (error "Calling shell without UNIX?")
	     #+excl (excl:shell string)
	     #+akcl (lisp:system string)
	     #+lispworks (foreign:call-system string "/bin/csh")
	     #+clisp (lisp:shell string)
	     #+cmu (extensions:run-program "/bin/csh" (list "-fc" string)
					   :output t)
	     #-(or excl akcl lispworks clisp cmu) 
	     (error "SHELL not implemented in this port!")))
    ;;;
    ;;; Now for the querying.  update keys with their default values here.
    ;;;
    
    (unless cm-directory
      (setf cm-directory
	    (query "Where is the Common Music source directory?:"
		   ;; Bug in gcl 1.1: (list (truename "") "~s") fails
		   ;; because of the tilde in the second string!
		   :default-value
		   (namestring #+excl (excl:current-directory)
			       #-excl (truename (make-string 0)))
		   :test-function #'directory-exists?
		   :reject-string "Directory ~A not found."
		   :feedback-string "Source directory set to ~A."
		   :help-string "Type the directory you installed CM in.")))
    (unless bd?
      (setf bin-directory
	    (query "Directory for compiled files:"
		   :default-value cm-directory
		   :test-function #'directory-exists?
		   :reject-string "Directory ~A not found."
		   :feedback-string "Binary directory set to ~A."
		   :help-string 
		   "Type the directory where compiled binaries will be kept."))
      (when (string= cm-directory bin-directory)
	(setf bin-directory nil)))
    (unless cms?
      (setf site-directory
	    (query "Directory for local site files:"
		   :default-value
		   (let ((c (elt cm-directory
				 (1- (length cm-directory)))))
		     (concatenate 'string cm-directory "site" (string c)))
		   :test-function #'directory-exists?
		   :reject-string "Directory ~A not found."
		   :help-string 
		   "Type the directory where site specific files will be kept."
		   :feedback-string "Site directory set to ~A.")))
    (unless site-directory
      (setf site-directory
	    (namestring
	     (make-pathname
	      :device (pathname-device cm-directory)
	      :host (pathname-host cm-directory)
	      :directory (append (pathname-directory
				  cm-directory) (list "site"))))))

    #-(or clos  pcl mcl clisp)
    (unless pd?
      (when (y-or-n-query "Do you use PCL?"
			  :feedback-string "Will~:[ not~;~] use PCL.")
	(setf pcl-directory
	      (query "Where is the PCL source directory?"
		     :test-function
		     #'(lambda (r)
			 (setf r (ensure-directory r))
			 (if (and (probe-file r)
				  (probe-file
				   (merge-pathnames "defsys.lisp" r)))
			   (values t r)
			   (values nil r)))
		     :reject-string
		     "~A: No such directory or \"~:*~adefsys.lisp\" not found."
		     :feedback-string "PCL directory set to ~A."
		     :help-string 
		     "The directory where PCL binaries are kept."))))
    #+(or pcl mcl clisp ansi-draft-cl-2)
    (when pd? (setf pcl-directory nil))
    
    (unless si?
      (setf save?
	    (query "Save Common Music?"
		   :default-value "If-No-Compile"
		   :test-function
		   #'(lambda (s)
		       (cond ((find s '("T" "Y" "YES" )
				    :test #'string-equal)
			      (values t t))
			     ((find s '("N" "NO" "NIL")
				    :test #'string-equal)
			      (values t nil))
			     ((find s '("IF-NO-COMPILE" ":IF-NO-COMPILE")
				    :test #'string-equal)
			      (values t ':if-no-compile))
			     (t nil)))
		   :reject-string
		   "~S is not YES, NO or IF-NO-COMPILE."
		   :feedback-string 
		   "Will~:[ not~;~] save Common Music."
		   :help-string
		   "Type YES, NO or IF-NO-COMPILE.~%~
		    YES           application file is saved.~%~
		    NO            application is not saved.~%~
		    IF-NO-COMPILE saves if no compile happens.~%")))
    
    (when (and save? (not cm-image-name))
      (let ((dir (or bin-directory cm-directory)))
	(setf cm-image-name
	      (namestring
	       (merge-pathnames
		(query "Save Common Music as?:"
		       :default-value
		       (namestring (merge-pathnames "cm" dir))
		       :test-function #'test-output-file-directory
		       :feedback-string "Save Common Music as: ~A."
		       :reject-string "~A: invalid directory component."
		       :help-string
		       "The application file this script will save")
		dir)))))
    (unless sy?
      (setf syntaxes
	    (query "List of syntaxes to use:"
		   :default-value "(:MIDI :CSOUND)"
		   :test-function
		   #'(lambda (r)
		       (if (string-equal r "(:MIDI :CSOUND)")
			 (values t '(:midi :csound))
			 (do ((s 0) (i t) (v ()) rev?)
			     ((not i) (values t (if rev? (nreverse v) v)))
			   (multiple-value-setq (i s)
			     (read-from-string r nil nil :start s))
			   (if (consp i)
			     (if (set-difference i %syntaxes%)
			       (return (values nil r))
			       (shiftf v i nil))
			     (if (keywordp i)
			       (if (member i %syntaxes%)
				 (progn (push i v) (setf rev? t))
				 (return (values nil r)))
			       (when i (return (values nil r))))))))
		   :reject-string "~S: bad syntax specification."
		   :feedback-string "Syntaxes set to ~S."
		   :help-string 
		   (format nil "The list of syntaxes to build into ~
				Common Music.~%List of any from: ~S"
			   %syntaxes%))))
    ;; check syntaxes even if not interactive.
    (unless (listp syntaxes) (setf syntaxes (list syntaxes)))
    (dolist (s syntaxes)
      (unless (find s %syntaxes%)
	(error "~S is not a Common Music syntax." s)))
    
    (when (and (not clmd?) (find :clm syntaxes))
      (setf clm-directory
	    (query "Where is the CLM binary directory?"
		   :test-function #'directory-exists?
		   :reject-string "Directory ~A not found."
		   :feedback-string "CLM source directory set to ~A.")))
    
    (when (and (not cmnd?) (find :cmn syntaxes))
      (setf cmn-directory
	    (query "Where is the CMN source directory?"
		   :test-function #'directory-exists?
		   :reject-string "Directory ~A not found."
		   :feedback-string "CMN directory set to ~A.")))
    #+mcl
    (unless cp?
      (setf capella
	    (y-or-n-query "Build Common Music with Capella?"
			  :feedback-string
			  "Will~:[ not~;~] include Capella.")))
    (unless ex?
      (notify "~%Extensions to be added:~%")
      (let ((ex '())
	    e)
	(block ext
	  (loop
	    (setf e (query "    Add"
			   :default-value "done"
			   :test-function
			   #'(lambda (r)
			       (if (string-equal r "done")
				 t 
				 (if (probe-file r)
				   r 
				   (values nil r))))
			   :reject-string 
			   "~S: bad file or directory specification."
			   :fresh-line nil))
	    (if (eq e t) (return-from ext) (push e ex))))
	(setf extensions (nreverse ex))
	(notify
	 (format nil "*** ~:[No extensions.~;Extensions:~:*~{~%  ~s~}~]"
		 extensions))))
    
    (when queried?
      (notify "~2%*** End Setup.~%")
      (let ((keys (format nil "~{~%~4T~s~40T~s~}"
			  (list :cm-directory cm-directory
				:cm-image-name cm-image-name
				:site-directory site-directory
				:bin-directory bin-directory
				:clm-directory clm-directory
				:cmn-directory cmn-directory
				#-(or mcl clisp ansi-draft-cl-2) 
				:pcl-directory
				#-(or mcl clisp ansi-draft-cl-2) 
				pcl-directory
				:save? save?
				:syntaxes (when syntaxes `',syntaxes)
				#+mcl :capella #+mcl capella
				:extensions 
				(when extensions `',extensions)))))
	(format *standard-output*
		"~&*** Building Common Music with parameters:~%~a"
		keys)
	(when (y-or-n-query
	       "Save these values to a local configuration script?")
	  (let ((f (query "Pathname to save the script in:"
			  :default-value
			  (namestring
			   (merge-pathnames #-(or dos aclpc win32)
					    "make-cm-local.lisp"
					    #+(or dos aclpc win32)
					    "MKCM-LCL.LSP"
					    site-directory))
			  :test-function #'test-output-file-directory
			  :reject-string "~s: invalid directory component."
			  :feedback-string "Saving ~A."))
		(m (concatenate 'string cm-directory 
				"build" delimiter 
                                 #-(or dos aclpc win32) "make-cm.lisp"
                                 #+(or dos aclpc win32) "MAKE-CM.LSP"
                                 )))
	    (when f
	      (setf f (namestring (merge-pathnames f site-directory)))
	      (with-open-file
		(s f :direction :output :if-does-not-exist :create
		   :if-exists :supersede)
		(multiple-value-bind (sec min hour dd mm yy x ds tz)
				     (decode-universal-time 
				      (get-universal-time))
		  (declare (ignore x tz))
		  (format
		   s
		   "~&;;; Created by make-cm.lisp on ~a/~a/~a at ~a:~a:~a~%~
		    (in-package #+cltl2 :cl-user #-cltl2 :user)~2%~
		    (let ((%make-cm-by-script% t))~2%  ~
		    (declare (special %make-cm-by-script%))~%  ~
		    (load ~s))~2%~
		    (make-cm ~a~%)~2%;;;~%;;; -*- EOF -*-~%"
		   mm dd yy (if ds (mod (1- hour) 24) hour) min sec 
		   m keys))))))
	(when (query "*** Ready to go!  Type <cr> to proceed or EXIT to exit:"
		     :test-function #'(lambda (r)
					(if (string-equal r "proceed")
					  (values t t)
					  (if (string-equal r "exit")
					    (return-from make-cm)
					    nil)))
		     :reject-string "~S is not <cr> or EXIT."
		     :default-value "proceed" )
	  (notify "~%*** Starting build process, this may take a while..."))))
    
    
    ;;;----------------------------------------------------------------------
    ;;;
    ;;; The Make
    ;;;
    ;;;----------------------------------------------------------------------
    
    
    #+mcl       
    (progn
      (ccl:require :resources)
      ;; let'ing these isn't enough in MCL3
      (setf *record-source-file*       t
	    *autoload-traps*           t
	    *save-doc-strings*         t
	    *save-local-symbols*       t
	    ccl::*save-arglist-info*   t
	    *fasl-save-local-symbols*  t)
      )
    
    
    #+clisp
    (unless (mismatch "IP" (machine-type) :end2 2)
      (pushnew ':sgi *features*))

    #+(and excl clos) ; excl 4.1 or 4.2
    (progn
      (pushnew :cltl2 *features*)
      (setf (excl:package-definition-lock (find-package ':common-lisp)) nil)
      (setf (excl:package-definition-lock (find-package ':clos)) nil))
 
    #+excl
    (progn
      (unless (eq excl::*current-case-mode* :case-insensitive-upper)
        (excl::set-case-mode :case-insensitive-upper))
      (when (find ':midi syntaxes)
        (require :process)))
    
    #+aclpc
    (progn 
      (setf acl:*warn-on-redefinition* nil)
      (setf acl:*warn-on-protected-function-redefinition* nil)
      (setf acl:*warn-on-protected-generic-function-redefinition* nil)
      (pushnew ':dos *features*))

    #+mcl
    (when capella (pushnew ':capella *features*))

    #+(and next (or excl clisp))
    (when (find ':midi syntaxes)
       (unless (probe-file (format nil "~Amidi/next/next-midi.o" cm-directory))
          (error "Forgot to make midi/next/next-midi.o! ")))

    #+(and sgi excl)
    (when (find ':midi syntaxes)
       (unless (probe-file (format nil "~Amidi/sgi/sgi-midi.so" cm-directory))
          (error "Forgot to make midi/sgi/sgi-midi.so! ")))

    ;; maybe load CLM
    #-clm
    (when clm-directory
      (let ((*default-pathname-defaults* (pathname clm-directory))
	    (loadfile (namestring (merge-pathnames 
				   "load-all.cl" clm-directory))))
	(if (probe-file loadfile)
	  (load loadfile)
	  (error "Can't find CLM load script: ~S."
		 loadfile))))
     ;; maybe load CMN
    #-cmn
    (when cmn-directory
      (let ((*default-pathname-defaults* (pathname cmn-directory)))
	(load (filename cmn-directory "cmn-all.lisp"))))
    
    ;; maybe load PCL. placed after possible CMN loading
    (when (and pcl-directory (not (find ':pcl *features*)))
      (let ((b (format nil "~Adefsys.~A" pcl-directory binary-type))
	    (s (format nil "~Adefsys.~A" pcl-directory source-type)))
	(if (probe-file b) (load b) (load s))
	(funcall (find-symbol "LOAD-PCL" :pcl))))
    
    (when (find ':pcl *features*)
      (set (find-symbol "*DEFCLASS-TIMES*" :pcl) '(compile load eval))
      (set (find-symbol "*DEFMETHOD-TIMES*" :pcl) '(load eval)))
    
    ;; load code walker if no CLM or ACL clos.
    #-(and excl :clos)
    (progn
      (unless (and (find :clm syntaxes)
		   (find-package :walker))
	(dofile cm-directory "extern" "walk"))
      (set (find-symbol "WALK-FORM-EXPAND-MACROS-P" ':walker) t))
    
    ;; maybe load loop
    (unless (find ':loop *features*)
      #+excl (require :loop)
      #+akcl (dofile cm-directory "extern" "loop")
      ;; can't use mcl's auto loading feature in saved images...
      #+mcl (progn (load "ccl:library;loop" :verbose t)
		   (pushnew ':loop *features*))
      #+clisp
      (handler-bind ((t #'(lambda (c)
			    (when (find-restart 'continue)
			      (invoke-restart 'continue)))))
	(dofile cm-directory "extern" "loop")))
    
    ;; load command interpreter. this is after clm because
    ;; we want to clobber clm's version, if it was loaded.
    
    (dofile cm-directory "extern" "tl") 
    ;(dofile cm-directory "extern" "defresource") 
    
    ;; load cm kernel      
    (let ((kern ' ("pkg" "resource" "utils1" "impl" "utils2" "classes" 
		   "scales" "items1" "items2"
		    "comptools"
		    "syntax" "edit" "select" "algo"
		   "io" "doobject" "archive" "copy" "files" 
		   )))
      (dolist (f kern) (dofile cm-directory "kernel" f)))
    
    ;; now set the system directory defvars declared in utils1.lisp
    (set (find-symbol "*COMMON-MUSIC-DIRECTORY*" :common-music)
	 cm-directory)
    
    (set (find-symbol "*CM-SITE-DIRECTORY*" :common-music)
	 site-directory)

    ;; maybe load midi syntax
    (when (find ':midi syntaxes)
      (dofile cm-directory "midi" "ml")
      #+(and excl next)
      (let ((o (filename cm-directory "midi" "next" "next-midi.o"))
            )
        
	    (if (probe-file o)
	      (load o :system-libraries '("sys_s"))
	      (error "Couldn't make midi/next/midi-next.o! Do it by hand."))
	    (dofile cm-directory "midi" "next" "acl-next-midi"))
      
      #+(and akcl |NeXT|)
      (progn
	;; copy next-midi.c to bin directory so compiler finds it.
	(when (and bin-directory
		   (not (string-equal bin-directory cm-directory)))
	  (let ((s (format nil "cp ~A ~A"
			   (filename cm-directory "midi" "next" 
				     "next-midi.c")          
			   (filename bin-directory "next-midi.c"))))
	    (shell s)))
	(dofile cm-directory "midi" "next" "gcl-next-midi")
	)
      
      #+(and clisp next)
      (dofile cm-directory "midi" "next" "clisp-next-midi")

      #+(or next |NeXT|) 
      (dofile cm-directory "midi" "next" "next-aux-midi")
      
      #+(and mcl (not powerpc)) (dofile cm-directory "midi" "mac" "mcl-midi")
      #+(and mcl powerpc) 
      (let ((to (merge-pathnames
                 "pmcl-midi.shlib"
                 (ccl::findfolder 32768    ;#$kOnSystemDisk
                                  ':|extn| ;#$kExtensionFolderType
                                  )))
            (from (filename cm-directory "midi" "mac" "pmcl-midi.shlib")))
        
        (when (or (not (probe-file to))
                  (< (file-write-date to) (file-write-date from)))
          (ccl:copy-file from to))
        (dofile cm-directory "midi" "mac" "pmcl-entry")
        (dofile cm-directory "midi" "mac" "pmcl-midi")
        )

      #+aclpc (dofile cm-directory "midi" "win" "winmidi")
      #+(and excl sgi)
      (progn
	(let ((o (filename cm-directory "midi" "sgi" "sgi-midi.so")))
	  (if (probe-file o)
	      (load o)
	    (error "Couldn't make midi/sgi/sgi-midi.so! Do it by hand."))
	  (dofile cm-directory "midi" "sgi" "sgi-midi")))
      
      (dofile cm-directory "midi" "md")
      (dofile cm-directory "midi" "mf")
      (dofile cm-directory "kernel" "midi"))

      (when (find :clm syntaxes)
	(dofile cm-directory "kernel" "clm"))
      (when (find :cmn syntaxes)
	(dofile cm-directory "kernel" "cmn"))
      (when (find :csound syntaxes)     
	(dofile cm-directory "kernel" "csound" ))
      (when (find :rt syntaxes) 
	(dofile cm-directory "kernel" "rt" ))
      (when (find :sgimix syntaxes)     
	(dofile cm-directory "kernel" "sgimix" ))
      (when (find :musicKit syntaxes)
	(dofile cm-directory "kernel" "mk")
	(unless (probe-file (filename cm-directory "site" "mk2stella"))
	  (warn "No site/mk2stella made. See site/README.")))
      (when (find ':m4c syntaxes)     
	(dofile cm-directory "kernel" "m4c" ))

    (let ((l '("cmd" "ask" "refs" "map" "stella")))
      (dolist (f l) (dofile cm-directory "stella" f))) 

    #+mcl
    (when capella
      (progn
	(dolist (f '("low" "defs" "controls" "menus" "table"
		     "listed" "infoed" "algoed" "ws" 
		     "enved" "output"))
	  (dofile cm-directory "capella" "mac" f))
	(let ((f (filename cm-directory "doc" "tutorials" "capella" 
			   "capella.cm")))
	  (unless (probe-file (make-pathname :type "fasl" :defaults f))
	    (compile-file f)))
	))
    (when extensions
      ;; let is not enough!
      (setf bin-directory nil)
      (let (#+mcl (ccl:*warn-if-redefine-kernel* nil))
        (dolist (f extensions)
          (dofile f))))

  ;; save image if possible    
  (if (or (eq save? t)
	  (and (eq save? ':if-no-compile) (not compiled?)))
    (let ((path cm-image-name))
      #+kcl (si:chdir (directory-namestring path)) 
      #+(or excl mcl aclpc lispworks)
      (progn (format t "~%Doing a full gc before save...")
	     #+excl (excl:gc t)
	     #+mcl  (ccl:gc)
	     #+aclpc (allegro:gc) 
	     #+lispworks (lispworks:mark-and-sweep 3)
	     )
      (format t "~%Saving Common Music in ~A" path)
      (funcall (find-symbol "SAVE-CM" :cm)  path))
    (when (eq save? ':if-no-compile) 
      (format t 
	      "~%Sources were compiled. To save Common Music, first~%~
	       exit lisp, then restart and reload make-cm.lisp."))
    )))

(eval-when (load eval)
  (unless (boundp '%make-cm-by-script%)
    (make-cm)))

;;;
;;; -*- EOF -*-


